﻿#include <iostream>
#include <list>
#include <string>

using namespace std;

enum category
{
	None = 0,
	Tanken = 1,
	Nahrungsmittel = 2,
	Reinigungsmittel = 3,
	Urlaub = 4,
	Haustiere = 5,
	Bekleidung = 6,
	Wohnen = 7,
	Versicherung = 8
};

class Invoice
{
private:
	int number_ = 0;
	double amount_;
	unsigned short month_ = 0;
	category category_ = None;
public:
	Invoice(int number, double amount, unsigned short month, category category) : number_(number), amount_(amount),
	                                                                              month_(month), category_(category)
	{
	}

	int get_number() const
	{
		return number_;
	}

	unsigned short get_month() const
	{
		return month_;
	}

	category get_category() const
	{
		return category_;
	}

	void print() const
	{
		cout << "Rechnung: " << number_ << endl;
		cout << "Betrag: " << amount_ << endl;
		cout << "Monat: " << month_ << endl;
		cout << "Kategorie: " << category_ << endl;
	}

		/**
	 * \brief Implements the equality operator.
	 */
	bool operator ==(const Invoice& e) const
	{
		return number_ == e.number_;
	}

	/**
	 * \brief Implements the inequality operator.
	 */
	bool operator !=(const Invoice& e) const { return !operator==(e); }

};

template <typename T>
T ask_question(const string& question)
{
	cout << question << endl;
	cout << "> ";
	T value;
	cin >> value;
	return value;
}

list<Invoice> invoices;

void create_invoice();

void output_invoices(list<Invoice> sorted_invoices);

void delete_invoice();

bool compare_category_ascending(const Invoice& first, const Invoice& second)
{
	return first.get_category() < second.get_category();
}

bool compare_month_ascending(const Invoice& first, const Invoice& second)
{
	return first.get_month() < second.get_month();
}

int main()
{
	auto end_program = false;
	while (!end_program)
	{
		system("cls");
		cout << "Ausgabenverwaltung (" << invoices.size() << " Rechnungen)" << endl << endl;
		cout << "Bitte w\x84hlen Sie eine Option:" << endl;
		cout << "1: Erfassung einer Rechnung (inkl. Abfrage der Rechnungsdaten)" << endl;
		cout << "2: Alle Ausgaben, sortiert nach Monat, ausgeben" << endl;
		cout << "3: Summe der Ausgaben pro Kategorie ausgeben" << endl;
		cout << "4: Rechnung l\x94schen" << endl;
		cout << endl;
		cout << "0: Programm beenden" << endl;
		const auto input = ask_question<int>("Ihre Eingabe:");

		switch (input)
		{
		case 1:
			create_invoice();
			break;
		case 2:
			{
				auto temp = list<Invoice>(invoices);
				temp.sort(compare_month_ascending);
				output_invoices(temp);
			}
			break;
		case 3:
			{
				auto temp = list<Invoice>(invoices);
				temp.sort(compare_category_ascending);
				output_invoices(temp);
			}
			break;
		case 4:
			delete_invoice();
			break;
		case 0:
			end_program = true;
			break;
		default:
			cout << "Unerlaubte Eingabe" << endl;
			system("pause");
		}
	}
	return 0;
}

category get_category();

void create_invoice()
{
	cout << "Eine neue Rechnung erstellen." << endl << endl;
	const auto invoice_no = invoices.empty() ? 1 : invoices.back().get_number() + 1;
	const auto amount = ask_question<double>("Bitte geben Sie den Betrag in Euro ein.");
	const auto month = ask_question<unsigned short>("Bitte geben Sie den Monat ein.");
	const auto category = get_category();
	const auto i = Invoice(invoice_no, amount, month, category);
	invoices.push_back(i);
}


void output_invoices(const list<Invoice> sorted_invoices)
{
	for (auto invoice : sorted_invoices)
	{
		invoice.print();
		cout << "--------------" << endl;
	}
	system("pause");
}

void delete_invoice()
{
	auto invoice_number = ask_question<int>("Bitte geben Sie die Rechnungsnummer ein");
	for (auto invoice : invoices)
	{
		if(invoice.get_number() == invoice_number)
		{
			invoices.remove(invoice);
			return;
		}
	}
	cout << "Rechnung nicht gefunden." << endl;
	system("pause");
}

category get_category()
{
	while (true)
	{
		cout << "Bitte w\x84hlen sie eine Kategorie aus:" << endl;
		cout << " 1: Tanken" << endl;
		cout << " 2: Nahrungsmittel" << endl;
		cout << " 3: Reinigungsmittel" << endl;
		cout << " 4: Urlaub" << endl;
		cout << " 5: Haustiere" << endl;
		cout << " 6: Bekleidung" << endl;
		cout << " 7: Wohnen" << endl;
		cout << " 8: Versicherung" << endl;
		auto cat = ask_question<int>("");
		if (cat > 0 && cat <= 8)
		{
			return static_cast<category>(cat);
		}
		cout << "Ung\x81ltige Eingabe" << endl << endl;
	}
}
